package com.informationkinematics.weblogic;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Title: TracerouteWrapperServlet</p> <p>
 *
 * Description: Platform independent webinterface for traceroute (or any other commandline
 * program, that expects an IP address as an input).</p> <p>
 *
 * Organization: Information Kinematics</p>
 *
 * @author    Copyright (c) 2002 by <a href="mailto:info@ente.limmat.ch"> information
 *      kinematics</a> under GNU General Public License
 * @created   20/04/2003
 * @version   0.5.2
 */

public class TracerouteWrapperServlet extends HttpServlet
{
    private String sTracerouteExecutablePath, sHTMLBeforeTracerouteOutput = "", sHTMLAfterTracerouteOutput, sNoResolvSwitch, sPlaceholderString;
    private File fPropertiesFile;
    private File fHTMLTemplate;
    private long lPropertiesFileLastModified;
    private long lTemplateLastModified;
    private String sContentType;
    private String sError = "";


    /**
     * On initialization read the servlet's inifile and HTML output template.
     *
     * @param config             Description of the Parameter
     * @throws ServletException
     */
    public void init( ServletConfig config )
        throws ServletException
    {
        fPropertiesFile = new File( config.getServletContext().getRealPath( "/" ) + "WEB-INF/classes/inifile.properties" );
        if ( !fPropertiesFile.exists() )
            throw new ServletException( "Properties file not found. Expected location is: "
                     + fPropertiesFile.getPath() );

        try
        {
            loadPropertiesFile();
            loadHTMLTemplate();
        } catch ( IOException ioe )
        {
            throw new ServletException( ioe );
        }
    }


    /**
     * Check the input from the HTML form, call a system process and put the output
     * into the prepared HTML page.
     *
     * @param req                Servlet's request object.
     * @param res                Servlet's response object.
     * @throws IOException
     * @throws ServletException
     */
    public void doPost( HttpServletRequest req, HttpServletResponse res )
        throws IOException, ServletException
    {

        boolean bNoResolv = !isEmpty( req.getParameter( "noresolv" ) );
        boolean bWasAlreadyNewLine = false;
        String sLine;
        String sHost = "";
        int index;

        // check for proper initialization
        if ( isEmpty( sError ) )
        {
            // reload the HTML template and the properties file
            dynamicReloader();

            sHost = req.getParameter( "host" );
            if ( isEmpty( sHost ) )
            {
                sHost = req.getRemoteAddr();
            }
            else if ( !isPossibleInetAddress( sHost ) )
            {
                sError = "Bad hostname or IP address. Please use only letters, digits,<br>"
                         + "dots and dashes within the input field.";
            }

            // replace the value in the host input field of the HTML output; like this the latest
            // typed hostname is preset
            index = sHTMLBeforeTracerouteOutput.indexOf( "name=\"host\"" ) + 11;
            sLine = " value=\"" + sHost + "\"";
            sHTMLBeforeTracerouteOutput = sHTMLBeforeTracerouteOutput.substring( 0, index )
                     + sLine
                     + sHTMLBeforeTracerouteOutput.substring
                    ( sHTMLBeforeTracerouteOutput.indexOf( ">", index ) );

            // add or remove "checked" for the noresolv checkbox
            index = sHTMLBeforeTracerouteOutput.indexOf( "name=\"noresolv\"" ) + 15;
            sHTMLBeforeTracerouteOutput = sHTMLBeforeTracerouteOutput.substring( 0, index )
                     + ( bNoResolv ? " checked" : "" )
                     + sHTMLBeforeTracerouteOutput.substring
                    ( sHTMLBeforeTracerouteOutput.indexOf( ">", index ) );
        }

        // send HTML header
        res.setContentType( sContentType );
        PrintWriter out = res.getWriter();
        out.println( sHTMLBeforeTracerouteOutput );

        // if there was an error so far, don't call the commandline process at all
        if ( !isEmpty( sError ) )
        {
            out.println( sError );
        }
        else
        {
            res.flushBuffer();

            Process p = Runtime.getRuntime().exec( sTracerouteExecutablePath + ( ( bNoResolv ) ? " " + sNoResolvSwitch : "" ) + " " + sHost );
            BufferedReader stdInput = new BufferedReader( new InputStreamReader( p.getInputStream() ) );
            // line by line output; because of a bug in windows's traceroute cut the empty lines
            while ( ( sLine = stdInput.readLine() ) != null )
            {
                if ( !isWindows() )
                {
                    out.println( sLine + "<br>" );
                    res.flushBuffer();
                }
                else
                {

                    if ( isEmpty( sLine ) && !bWasAlreadyNewLine )
                    {
                        out.println( "<br>" );
                        bWasAlreadyNewLine = true;
                    }
                    else
                    {
                        out.println( sLine );
                        res.flushBuffer();
                        bWasAlreadyNewLine = false;
                    }

                }
            }
        }
        out.println( !isEmpty( sHTMLAfterTracerouteOutput ) ? sHTMLAfterTracerouteOutput : "" );
        out.close();
		sError = "";
    }


    /**
     * Check wether a string is empty or null.
     *
     * @param sTestString  The string to test.
     * @return             Returns true if the test string is empty or null.
     */
    private boolean isEmpty( String sTestString )
    {
        return ( sTestString == null || sTestString.equals( "" ) );
    }


    /**
     * Returns the type of the operation system, as follows:<br>
     * true: Windows family<br>
     * false: Others<br>
     *
     *
     * @return   boolean - true if OS name contains "Win", else false
     */
    private boolean isWindows()
    {
        return ( System.getProperty( "os.name" ).toLowerCase().indexOf( "win" ) != -1 );
    }


    /**
     * Checks the given string for "undecent" characters, if none are found the string
     * might be an IP address. Anything other than letters, digits, dots and dashes
     * are regarded as undecent.
     *
     * @param address                        The address to test.
     * @return                               Returns true, if the given string could
     *      be an IP address or hostname.
     * @exception IndexOutOfBoundsException
     */
    private boolean isPossibleInetAddress( String address )
        throws IndexOutOfBoundsException
    {
        char chCurrent;
        int index = -1;
        boolean isPossibleInetAddress = true;

        while ( isPossibleInetAddress == true && index < address.length() - 1 )
        {
            index += 1;
            chCurrent = address.charAt( index );
            if ( !( Character.isLetterOrDigit( chCurrent ) || chCurrent == '.'
                     || chCurrent == '-' || chCurrent == '_' || chCurrent == '@' ) )
            {
                isPossibleInetAddress = false;
            }
        }
        return isPossibleInetAddress;
    }


    /**
     * Load the properties file and read the initial parameters, check them for plausibility.
     *
     * @throws IOException
     */
    private void loadPropertiesFile()
        throws IOException
    {
        try
        {
            Properties props = new Properties();
            props.load( new FileInputStream( fPropertiesFile ) );

            sTracerouteExecutablePath = props.getProperty( "TraceroutePath" );
            String sHTMLTemplate = props.getProperty( "HTMLTemplate" );
            String sCharset = props.getProperty( "Charset" );
            sPlaceholderString = props.getProperty( "PlaceholderString" );
            sNoResolvSwitch = props.getProperty( "NoResolvSwitch" );

            File fTracerouteExecutable = new File( sTracerouteExecutablePath );
            fHTMLTemplate = new File( sHTMLTemplate );

            if ( isEmpty( sTracerouteExecutablePath ) )
            {
                sError = "<b>Wrong Configuration:</b> '" + fPropertiesFile.getPath()
                         + "'<br> must contain the path to an executable program as the argument for 'TraceroutePath'.";
            }
            else if ( !fTracerouteExecutable.exists() )
            {
                sError = "<b>File not found:</b> The given path to the executable program in<br> '"
                         + fPropertiesFile.getPath() + "'<br> doesn't exist. Please check the variable 'TraceroutePath'.";
            }
            else if ( isEmpty( sHTMLTemplate ) )
            {
                sError = "<b>Wrong Configuration:</b> '" + fPropertiesFile.getPath()
                         + "'<br> must contain the path to a HTML template as the argument for 'HTMLTemplate'.";
            }
            else if ( !fHTMLTemplate.exists() )
            {
                sError = "<b>File not found:</b> The given path to the HTML template in<br> '"
                         + fPropertiesFile.getPath() + "'<br> doesn't exist. Please check the variable 'HTMLTemplate'.";
            }
            else if ( isEmpty( sPlaceholderString ) )
            {
                sError = "<b>Wrong Configuration:</b> No Placeholder defined in <br> '" + fPropertiesFile.getPath() + "',<br>"
                         + "variable 'PlaceholderString'. Like this I don't know what to replace in the HTML template.";
            }
            else if ( isEmpty( sCharset ) )
            {
                sContentType = "Charset = iso-8859-1";
            }

            sContentType = "text/html; charset=" + sCharset;
            lPropertiesFileLastModified = fPropertiesFile.lastModified();

        } catch ( FileNotFoundException fnfe )
        {
            sError = "<html><br><br><b>File not found!</b> " + fnfe.getMessage() + "<p>"
                     + "The properties file was may be moved from its original location; however it's expected location is: "
                     + fPropertiesFile.getPath() + "<br></html>";
        }
    }


    /**
     * Load a template for HTML output.
     *
     * @exception IOException  Description of the Exception
     */
    private void loadHTMLTemplate()
        throws IOException
    {
        sHTMLBeforeTracerouteOutput = "";

        BufferedReader br = new BufferedReader( new FileReader( fHTMLTemplate ) );
        String sLine;
        boolean bPlaceholderFound = false;
        int iIndex;

        while ( ( sLine = br.readLine() ) != null )
        {
            if ( !bPlaceholderFound && ( ( iIndex = sLine.indexOf( sPlaceholderString ) ) >= 0 ) )
            {
                sHTMLBeforeTracerouteOutput += sLine.substring( 0, iIndex );
                sHTMLAfterTracerouteOutput = sLine.substring( iIndex + sPlaceholderString.length() ) + "\n";
                bPlaceholderFound = true;
            }
            else if ( !bPlaceholderFound )
            {
                sHTMLBeforeTracerouteOutput += sLine + "\n";
            }
            else
            {
                sHTMLAfterTracerouteOutput += sLine + "\n";
            }
        }

        if ( !bPlaceholderFound )
        {
            sError = "<p><b>Error in HTML template:</b> The PlaceholderString '"
                     + ( sPlaceholderString.startsWith( "<" ) ? "&lt;" : sPlaceholderString.substring( 0, 1 ) )
                     + sPlaceholderString.substring( 1 )
                     + "' definded in<br> '" + fPropertiesFile.getPath() + "'<br>"
                     + "is missing in the HTML template <br>'" + fHTMLTemplate.getPath() + "'.";
        }

        lTemplateLastModified = fHTMLTemplate.lastModified();
    }


    /**
     * Reload the properties file or HTML template if needed.
     *
     * @throws IOException  Description of the Exception
     */
    private void dynamicReloader()
        throws IOException
    {
        // dynamic properties file reloading
        if ( fPropertiesFile.lastModified() != lPropertiesFileLastModified )
        {
            loadPropertiesFile();
        }

        // dynamic template reloading
        if ( fHTMLTemplate.lastModified() != lTemplateLastModified )
        {
            loadHTMLTemplate();
        }
    }

}
